unit uAdvRTF;

{$INCLUDE 'RTFConv.inc'}
{.DEFINE STYLES}
{$DEFINE METADATA}

interface

uses
  SysUtils, Classes,
  {$IFDEF STYLES} uRtfStyles, {$ENDIF}
  {$IFDEF METADATA} uRtfMetadata, {$ENDIF}
  uRTF, uRtfTokens;

type
  TAdvRtf = class(TRtf)
    private
      {$IFDEF STYLES}
      fStyles: TRtfStyleList;
      {$ENDIF}
      {$IFDEF METADATA}
      fMetadata: TRtfMetadata;
      {$ENDIF}
    protected
      procedure Build; virtual;
    {$IFDEF STYLES}
    protected
      procedure BuildStyleList; virtual;
      property Styles: TRtfStyleList read fStyles;
    {$ENDIF}
    {$IFDEF METADATA}
    protected
      procedure BuildMetadata; virtual;
      property Metadata: TRtfMetadata read fMetadata;
    {$ENDIF}
    public
      constructor Create; override;
      destructor Destroy; override;
      procedure Clear; override;
      procedure LoadFromStream(Stream: TStream); override;
    end;


implementation

{ TAdvRtf }

constructor TAdvRtf.Create;
begin
  inherited;
  {$IFDEF STYLES}
  fStyles := TRtfStyleList.Create;
  {$ENDIF}
  {$IFDEF METADATA}
  fMetadata := TRtfMetadata.Create;
  {$ENDIF}
end;

destructor TAdvRtf.Destroy;
begin
  {$IFDEF STYLES}
  FreeAndNil(fStyles);
  {$ENDIF}
  {$IFDEF METADATA}
  FreeAndNil(fMetadata);
  {$ENDIF}
  inherited;
end;

procedure TAdvRtf.Clear;
begin
  {$IFDEF STYLES}
  fStyles.Clear;
  {$ENDIF}
  {$IFDEF METADATA}
  fMetadata.Clear;
  {$ENDIF}
  inherited;
end;

procedure TAdvRtf.LoadFromStream(Stream: TStream);
begin
  inherited;
  Build;
end;

procedure TAdvRtf.Build;
begin
  {$IFDEF STYLES}
  BuildStyleList;
  {$ENDIF}
  {$IFDEF METADATA}
  BuildMetadata;
  {$ENDIF}
end;

{$IFDEF STYLES}
procedure TAdvRtf.BuildStyleList;
var i, n, Depth: integer;
    Style: TRtfStyle;
begin
  Styles.Clear;
  Style := nil;
  n := Pred(Tokens.Count);
  i := 0;
  // Find the stylesheet
  while (i <= n) do
    with Tokens[i] do
      if (TokenType = rtfControlWord) and (Token = '\stylesheet') then
        begin
        Inc(i);
        Depth := 1;
        // Parse all available styles in the stylesheet.
        while (i <= n) and (Depth > 0) do
          with Tokens[i] do
            if TokenType = rtfGroupStart then
              begin
              Inc(i);
              Inc(Depth);
              if Depth = 2 then
                begin
                Style := TRtfStyle.Create(i);
                Styles.Add(Style);
                end;
              end
            else if TokenType = rtfGroupEnd then
              begin
              Inc(i);
              Dec(Depth);
              if (Depth = 1) and (Style <> nil) then
                Style := nil;
              end
            else if (TokenType = rtfControlSymbol) and (Token = '\*') then
              SkipGroup(i)
            else if (Style = nil) then
              Inc(i)
            // Here we start processing Style definitions. Style is always filled in
            else if (TokenType = rtfPlainText) and (Token = ';') then
              Style := nil
            else
              case TokenType of
                rtfControlWordWithParam:
                  begin
                  end;
                rtfControlWord:
                  begin
                  end;
                rtfControlSymbol:
                  begin
                  end;
                end;

        if (Depth > 0) or (Style <> nil) then
          Raise ERtfError.Create('Unfinished stylesheet encountered.');
        // There won't be any more stylesheets, as per RTF definition
        Break;
        end
      else if (TokenType = rtfControlSymbol) and (Token = '\*') then
        SkipGroup(i)
      else
        Inc(i);
end;
{$ENDIF}

{$IFDEF METADATA}
procedure TAdvRtf.BuildMetadata;
begin

end;
{$ENDIF}

end.
