unit uRtfStyles;

{$INCLUDE 'RTFConv.inc'}

interface

uses
  SysUtils, Classes,
  uRtfTokens;

type
  TRtfStyleType = (stParagraph, stSection, stCharacter);

  TRtfStyle = class;
  TRtfStyleList = class;

  TRtfStyle = class
    private
      fStyleStart: integer;
      fStyleType: TRtfStyleType;
      fNumber: integer;
      fAdditive: boolean;
      fBasedOnStyle, fNextStyle: integer;
    protected
    public
      constructor Create(AStyleStart: integer); virtual;
      destructor Destroy; override;
      function ProcessStyleProperty(ARtfToken: TRtfToken): boolean; virtual;
    end;

  TRtfStyleList = class
    private
      fList: TList;
      function GetCount: integer;
      function GetItems(Index: integer): TRtfStyle;
    protected
    public
      constructor Create; virtual;
      destructor Destroy; override;
      procedure Clear; virtual;
      function Add(Item: TRtfStyle): integer; virtual;
      procedure Append(Index: integer; Item: TRtfStyle); virtual;
      procedure Insert(Index: integer; Item: TRtfStyle); virtual;
      property Items[Index: integer]: TRtfStyle read GetItems; default;
      property Count: integer read GetCount;
    end;

implementation

{ TRtfStyleList }

constructor TRtfStyleList.Create;
begin
  inherited Create;
  fList := TList.Create;
end;

destructor TRtfStyleList.Destroy;
begin
  Clear;
  FreeAndNil(fList);
  inherited;
end;

procedure TRtfStyleList.Clear;
var i: integer;
begin
  for i := 0 to Pred(fList.Count) do
    TObject(fList[i]).Free;
  fList.Clear;
end;

function TRtfStyleList.GetCount: integer;
begin
  Result := fList.Count;
end;

function TRtfStyleList.GetItems(Index: integer): TRtfStyle;
begin
  Result := TRtfStyle(fList[Index]);
end;

function TRtfStyleList.Add(Item: TRtfStyle): integer;
begin
  Result := fList.Add(Item);
end;

procedure TRtfStyleList.Append(Index: integer; Item: TRtfStyle);
begin
  if Succ(Index) < fList.Count then
    fList.Insert(Succ(Index), Item)
  else
    fList.Add(Item);
end;

procedure TRtfStyleList.Insert(Index: integer; Item: TRtfStyle);
begin
  fList.Insert(Index, Item);
end;

{ TRtfStyle }

constructor TRtfStyle.Create(AStyleStart: integer);
begin
  inherited Create;
  fStyleStart := AStyleStart;
  fBasedOnStyle := 222;
  fNextStyle := 0;
end;

destructor TRtfStyle.Destroy;
begin
  inherited;
end;

function TRtfStyle.ProcessStyleProperty(ARtfToken: TRtfToken): boolean;
begin
  Result := False;
  with ARtfToken do
    if TokenType in [rtfControlWord, rtfControlWordWithParam] then
      begin
      if Token = '\s' then
        begin
        Result := True;
        fStyleType := stParagraph;
        fNumber := StrToInt(Parameter);
        end
      else if Token = '\ds' then
        begin
        Result := True;
        fStyleType := stSection;
        fNumber := StrToInt(Parameter);
        end
      else if Token = '\cs' then
        begin
        Result := True;
        fStyleType := stCharacter;
        fNumber := StrToInt(Parameter);
        end
      else if Token = '\additive' then
        begin
        Result := True;
        fAdditive := True;
        end
      else if Token = '\sbasedon' then
        begin
        Result := True;
        fBasedOnStyle := StrToInt(Parameter);
        end
      else if Token = '\snext' then
        begin
        Result := True;
        fNextStyle := StrToInt(Parameter);
        end


      end;
end;

end.
