unit uRtfTokens;

{$INCLUDE 'RTFConv.inc'}

interface

uses
  SysUtils, Classes;
  
type
  TRtfTokenType = (rtfError, rtfPlainText, rtfBinaryData, rtfControlWord, rtfControlWordWithParam, rtfControlSymbol, rtfGroupStart, rtfGroupEnd);

  TRtfToken = class;
  TRtfTokenList = class;

  TRtfToken = class
    private
    protected
    public
      Token, Parameter: string;
      TokenType: TRtfTokenType;
      constructor Create(ATokenType: TRtfTokenType; const AToken, AParameter: string); overload; virtual;
      constructor Create; overload; virtual;
    end;

  TRtfTokenList = class
    private
      fList: TList;
      fInserted: boolean;
      function GetCount: integer;
      function GetItems(Index: integer): TRtfToken;
    protected
    public
      constructor Create; virtual;
      destructor Destroy; override;
      procedure Clear; virtual;
      function Add(Item: TRtfToken): integer; virtual;
      procedure Append(Index: integer; Item: TRtfToken); virtual;
      procedure Insert(Index: integer; Item: TRtfToken); virtual;
      property Items[Index: integer]: TRtfToken read GetItems; default;
      property Count: integer read GetCount;
      property Inserted: boolean read fInserted write fInserted;
    end;

implementation

{ TRtfTokenList }

constructor TRtfTokenList.Create;
begin
  inherited Create;
  fList := TList.Create;
end;

destructor TRtfTokenList.Destroy;
begin
  Clear;
  FreeAndNil(fList);
  inherited;
end;

procedure TRtfTokenList.Clear;
var i: integer;
begin
  for i := 0 to Pred(fList.Count) do
    TObject(fList[i]).Free;
  fList.Clear;
end;

function TRtfTokenList.GetCount: integer;
begin
  Result := fList.Count;
end;

function TRtfTokenList.GetItems(Index: integer): TRtfToken;
begin
  Result := TRtfToken(fList[Index]);
end;

function TRtfTokenList.Add(Item: TRtfToken): integer;
begin
  Result := fList.Add(Item);
end;

procedure TRtfTokenList.Insert(Index: integer; Item: TRtfToken);
begin
  fList.Insert(Index, Item);
  fInserted := True;
end;

procedure TRtfTokenList.Append(Index: integer; Item: TRtfToken);
begin
  if Succ(Index) < Count then
    Insert(Succ(Index), Item)
  else
    Add(Item);
end;

{ TRtfToken }

constructor TRtfToken.Create(ATokenType: TRtfTokenType; const AToken, AParameter: string);
begin
  inherited Create;
  TokenType := ATokenType;
  Token := AToken;
  Parameter := AParameter;
end;

constructor TRtfToken.Create;
begin
  Create(rtfPlainText, '', '');
end;

end.
